# **AntiBP3**
A tool for predicting and designing Antibacterial peptides for three groups of bacteria using the sequence information.
## Introduction
Antibacterial peptides (ABPs) are specific cationic AMPs against various bacterial species that disturb the bacterial cell membrane with different mechanisms. The ABPs can also hamper intracellular processes of the bacterial pathogen in different ways, such as modulating the enzymatic activity, protein degradation & synthesis and nucleic acid synthesis, which gives an advantage over traditional antibiotics in developing resistance. Hence, develop an Antibacterial prediction tool for gram-positive, gram-negative and gram-variable bacteria.
AntiBP3 is also available as a web server at https://webs.iiitd.edu.in/raghava/antibp3. Please read/cite the content about the AntiBP3 for complete information, including the algorithm behind the approach.


## Installation
To install the package, type the following command:
```
pip install antibp3
```

## Minimum USAGE
To know about the available option for the CLI tool, type the following command:
```
antibp3 -h
```

This will predict if the submitted sequences are Antibacterial or non-Antibacterial for the chosen class of bacteria. It will use other parameters by default. It will save the output in "outfile.csv" in CSV (comma-separated variables).

## Full Usage
```
usage: antibp3 [-h] 
                       [-i INPUT 
                       [-o OUTPUT]
                       [-s {1,2,3}]
		       [-j {1,2,3,4,5}]
		       [-t THRESHOLD]
                       [-e EVAL]
		       [-d {1,2}]
		       [-wd WORKING]
```
```
Please provide the following arguments for the successful run

Optional arguments:
  -h, --help            show this help message and exit
  -i INPUT, --input INPUT
                        Input: protein or peptide sequence(s) in FASTA format or single sequence per line in single letter code
  -o OUTPUT, --output OUTPUT
                        Output: File for saving results by default outfile.csv
  -s {1,2,3}, --source {1,2,3}
                        Source: 1:GP ABPs, 2:GN ABPs, 3:GV ABPs by default 1
 -j {1,2,3,4,5}, --job {1,2,3,4,5}
                        Job Type: 1:Predict, 2:Design, 3:BLAST Search 4:Motif Scan, 5:Protein Scan ; by default 1
  -t THRESHOLD, --threshold THRESHOLD
                        Threshold: Value between 0 to 1 by default 0.5 for GP ABPs, 0.45 for GN ABPs and 0.51 for GV ABPs   
  -e EVAL, --eval EVAL  E-value for Blast search (Blast Search only), by default 0.01 for GP ABPs, 0.01 for GN ABPs and 0.001 for GV ABPs
  -w {8,9,10,11,12,13,14,15,16,17,18,19,20}, --winleng {8,9,10,11,12,13,14,15,16,17,18,19,20}
                        Window Length: 8 to 20 (scan mode only), by default 8
  -d {1,2}, --display {1,2}
                        Display: 1:ABPs only, 2: All peptides, by default 1
  -wd WORKING, --working WORKING
                        Working Directory: Location for writing results
```

**Input File:** It allows users to provide input in the FASTA format.

**Output File:** Program will save the results in the CSV format; if the user does not provide the output file name, it will be stored in "outfile.csv".

**Source:** User should provide sources 1,2, and 3; by default, it's 1 for GP ABPs.

**Working Directory:** This option allows users to set the working directory in which they want to get the output files.

**Job:** User is allowed to choose between three different modules, such as 1 for prediction, 2 for Designing, 3 for Blast scan, 4 for Motif scan and 5 for Protein scan; by default, it's 1.

**Threshold:** User should provide a threshold between 0 and 1; by default, 0.5 for GP ABPs, 0.45 for GN ABPs and 0.51 for GV ABPs

**Window length**: User can choose any pattern length between 8 and 20 in long sequences. This option is available for only scanning module.

**e-value:** User is allowed to choose an e-value for Blast search between  0.0001 to 1e-20; by default, 0.01 for GP ABPs, 0.01 for GN ABPs and 0.001 for GV ABPs

**Display type:** This option allow users to fetch either only Antibacterial peptides by choosing option 1 or prediction against all peptides by choosing option 2.

